import { transType } from './transform';
import { getJsonProperty, hasAnyNullOrUndefined, isArray, isObject, setProperty } from './utils';
export function mapperProperty(value, typeName) {
    var metadata = {
        typeName: typeName,
        localKey: value
    };
    return setProperty(metadata);
}
export function deepMapperProperty(value, Clazz) {
    var metadata = {
        localKey: value,
        Clazz: Clazz
    };
    return setProperty(metadata);
}
export function filterMapperProperty(value, filter) {
    var metadata = {
        localKey: value,
        filter: filter
    };
    return setProperty(metadata);
}
export function deserialize(Clazz, json) {
    if (hasAnyNullOrUndefined(Clazz, json)) {
        throw new Error('(type-json-mapper)deserialize：missing Clazz or json');
    }
    if (!isObject(json)) {
        throw new Error('(type-json-mapper)deserialize：json is not a object');
    }
    var result = {};
    var instance = new Clazz();
    var keys = Object.keys(instance);
    result = instance;
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        var value = json[key];
        var metaObj = {};
        metaObj = getJsonProperty(instance, key);
        if (typeof metaObj === 'undefined') {
            metaObj = {};
        }
        var typeName = metaObj.typeName, localKey = metaObj.localKey;
        if (!['', 0, undefined].includes(localKey)) {
            value = json[localKey];
            if (typeof value !== 'undefined') {
                value = transType(value, typeName);
            }
        }
        var filter = metaObj.filter;
        if (typeof filter === 'function') {
            var tempVal = filter(value);
            if (typeof tempVal !== 'undefined') {
                value = tempVal;
            }
        }
        var childClazz = metaObj.Clazz;
        if (typeof childClazz !== 'undefined') {
            if (isObject(value)) {
                value = deserialize(childClazz, value);
            }
            if (isArray(value)) {
                value = deserializeArr(childClazz, value);
            }
        }
        result[key] = value;
    }
    return result;
}
export function deserializeArr(Clazz, list) {
    if (hasAnyNullOrUndefined(Clazz, list)) {
        throw new Error('(type-json-mapper)deserializeArr：missing Clazz or list');
    }
    return list.map(function (ele) { return deserialize(Clazz, ele); });
}
